@@ -13,7 +13,6 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
-#include <sys_defs.h>
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -23,15 +22,6 @@
 #include <unistd.h>
 #include <syslog.h>
 #include <sys/stat.h>
-/* global library */
-#include <mymalloc.h>
-#include <stringops.h>
-#include <msg_syslog.h>
-#include <msg.h>
-#include <mail_task.h>
-#include <mail_conf.h>
-#include <mail_params.h>
-#include <mail_dict.h>
 
 #include <db.h>
 
@@ -52,10 +42,9 @@
  */
 #define AUTO_RECORD_LIFE_SECS (5 * 3600) /* 5 hours */
 
-#define CONF_DB_HOME PACKAGE"_directory"
 #define DEF_DB_HOME DATA_STATE_DIR"/"PACKAGE
 
-#define DB_FILE_NAME "triplets"
+#define DB_FILE_NAME "triplets.db"
 #define SEP '\000'
 
 #define prefixp(s,p) (!strncmp((s),(p),sizeof(p)-1))
@@ -72,6 +61,10 @@ static const char str_esmtp[] = "ESMTP";
 static const char str_rcpt[] = "RCPT";
 static const char str_action[] = "action=";
 
+static const char *progname;
+static const char *db_home = DEF_DB_HOME;
+static int opt_verbose = 0;
+
 static DB_ENV *dbenv = 0;
 static DB *db = 0;
 
@@ -91,26 +84,25 @@ static int debug_me = 0;
 static void
 log_db_error(const char *msg, int error)
 {
-    msg_error("%s: %s", msg, db_strerror(error));
+    syslog(LOG_ERR, "%s: %s", msg, db_strerror(error));
 }
 
 static void
 db_errcall_fcn(const DB_ENV *dbenv, const char *errpfx, const char *msg)
 {
-    msg_error("%s: %s", errpfx ? errpfx : "Berkeley DB", msg);
+    syslog(LOG_ERR, "%s: %s", errpfx ? errpfx : "Berkeley DB", msg);
 }
 
 static int
 prepare_env()
 {
     int rc;
-    const char *home = mail_conf_eval(get_mail_conf_str(CONF_DB_HOME, DEF_DB_HOME, 0, 0));
     rc = db_env_create(&dbenv, 0);
     if (rc)
 	log_db_error("db_env_create", rc);
     else {
 	dbenv->set_errcall(dbenv, db_errcall_fcn);
-	rc = dbenv->open(dbenv, home,
+	rc = dbenv->open(dbenv, db_home,
 			 DB_INIT_CDB | DB_INIT_MPOOL | DB_CREATE, 0);
 	if (rc)
 	    log_db_error("dbenv->open", rc);
@@ -142,22 +134,25 @@ initialize()
     int major, minor, patch;
     version = db_version(&major, &minor, &patch);
     if (DB_VERSION_MAJOR != major || DB_VERSION_MINOR != minor) {
-	msg_fatal("This daemon was compiled with " DB_VERSION_STRING " (%d.%d.%d) definitions "
-		  "but it is linked with %s (%d.%d.%d).  This will not work!  "
-		  "Check that the version of the developpement files for Berkeley DB "
-		  "match the version that was linked in your Postfix program.",
-		  DB_VERSION_MAJOR, DB_VERSION_MINOR, DB_VERSION_PATCH,
-		  version, major, minor, patch);
+	syslog(LOG_ERR,
+	       "This daemon was compiled with " DB_VERSION_STRING " (%d.%d.%d) definitions "
+	       "but it is using %s (%d.%d.%d).  This will not work!  "
+	       "Check that the version of the developpement files for Berkeley DB "
+	       "match the version that used.",
+	       DB_VERSION_MAJOR, DB_VERSION_MINOR, DB_VERSION_PATCH,
+	       version, major, minor, patch);
 	abort();
     }
-    if (DB_VERSION_PATCH != patch && (msg_verbose || debug_me))
-	msg_info("Compiled with " DB_VERSION_STRING " (%d.%d.%d) definitions.  "
-		 "Running with %s (%d.%d.%d).",
-		 DB_VERSION_MAJOR, DB_VERSION_MINOR, DB_VERSION_PATCH,
-		 version, major, minor, patch);
+    if (DB_VERSION_PATCH != patch && (opt_verbose || debug_me))
+	syslog(LOG_INFO,
+	       "Compiled with " DB_VERSION_STRING " (%d.%d.%d) definitions.  "
+	       "Running with %s (%d.%d.%d).",
+	       DB_VERSION_MAJOR, DB_VERSION_MINOR, DB_VERSION_PATCH,
+	       version, major, minor, patch);
     else if (debug_me)
-	msg_info("This daemon was compiled with " DB_VERSION_STRING " (%d.%d.%d) definitions.",
-		 DB_VERSION_MAJOR, DB_VERSION_MINOR, DB_VERSION_PATCH);
+	syslog(LOG_DEBUG,
+	       "This daemon was compiled with " DB_VERSION_STRING " (%d.%d.%d) definitions.",
+	       DB_VERSION_MAJOR, DB_VERSION_MINOR, DB_VERSION_PATCH);
     dbdata.data = &triplet_data;
     dbdata.size = sizeof triplet_data;
     dbdata.ulen = sizeof triplet_data;
@@ -187,9 +182,18 @@ cleanup()
 	dbenv = 0;
 	if (rc)
 	    log_db_error("DB_ENV close", rc);
+	/* Clean up the environment so that upgrading is easier. */
+	rc = db_env_create(&dbenv, 0);
+	if (rc)
+	    log_db_error("db_env_create failed during cleanup", rc);
+	else {
+	    rc = dbenv->remove(dbenv, db_home, 0);
+	    if (rc)
+		syslog(LOG_WARNING, "db_env->remove failed", rc);
+	}
     }
     if (debug_me)
-	msg_info("Cleaned");
+	syslog(LOG_DEBUG, "Cleaned");
 }
 
 static void
@@ -197,7 +201,7 @@ fatal(const char *msg)
 {
     int err = errno;;
     cleanup();
-    msg_fatal("%s: %s", msg, db_strerror(err));
+    syslog(LOG_ERR, "fatal: %s: %s", msg, db_strerror(err));
     abort();
 }
 
@@ -266,9 +270,9 @@ safe_writes(int fd, const char *s)
 	if (n > 0)
 	    len -= n;
 	else if (n == 0)
-	    msg_error("Retrying due to empty write");
+	    syslog(LOG_ERR, "Retrying due to empty write");
 	else if (errno == EINTR && debug_me)
-	    msg_info("Retrying write: %s", strerror(errno));
+	    syslog(LOG_DEBUG, "Retrying write: %s", strerror(errno));
     }
 }
 
@@ -280,17 +284,8 @@ find_attribute(const char *name)
     size_t nlen;
     nlen = strlen(name);
     for (p = policy_request; *p != '\n'; p = strchr(p, '\n') + 1)
-	if (strncmp(name, p, nlen) == 0 && p[nlen] == '=') {
-/*
-	    if (debug_me) {
-		char *e = strchr(p, '\n');
-		*e = 0;
-		msg_info("Attribute %s", p);
-		*e = '\n';
-	    }
-*/
+	if (strncmp(name, p, nlen) == 0 && p[nlen] == '=')
 	    return p + nlen + 1;
-	}
     return NULL;
 }
 
@@ -341,8 +336,9 @@ read_policy_request(int in)
 	    else
 		policy_request_size = BUFSIZ;
 	    if (debug_me)
-		msg_info("allocate %u bytes for request buffer",
-			 policy_request_size);
+		syslog(LOG_DEBUG,
+		       "allocate %u bytes for request buffer",
+		       policy_request_size);
 	    policy_request = xrealloc(policy_request, policy_request_size);
 	}
 	wanted = policy_request_size - policy_request_fill;
@@ -390,7 +386,7 @@ process_smtp_rcpt()
     if (triplet_data.pass_count == 0
 	&& triplet_data.access_time - triplet_data.create_time > AUTO_RECORD_LIFE_SECS) {
 	if (debug_me)
-	    msg_info("expired record");
+	    syslog(LOG_DEBUG, "expired record");
 	triplet_data.create_time = triplet_data.access_time;
     }
     /* Block inbound mail that is from a previously unknown (ip, from, to) triplet */
@@ -405,20 +401,30 @@ process_smtp_rcpt()
 }
 
 int
-main(int argc, char **argv)
+main(int argc, const char **argv)
 {
+    const char **parg;
     int rc;
-    if (getenv(CONF_ENV_VERB))
-        msg_verbose = 1;
-    if (getenv(CONF_ENV_DEBUG))
-        debug_me = 1;
-    var_procname = mystrdup(basename(argv[0]));
-    set_mail_conf_str(VAR_PROCNAME, var_procname);
-    msg_syslog_init(mail_task(var_procname), LOG_PID, LOG_FACILITY);
-    if (msg_verbose)
-        msg_info("daemon started");
-    mail_conf_read();
-    mail_dict_init();
+    progname = strrchr(argv[0], '/');
+    if (progname)
+	progname++;
+    else
+	progname = argv[0];
+    openlog(progname, LOG_PID, LOG_DAEMON);
+    for (parg = argv; *parg; parg++) {
+	if (strcmp("-d", *parg) == 0)
+	    debug_me = 1;
+	else if (strcmp("-v", *parg) == 0)
+	    opt_verbose++;
+	else if (strcmp("-h", *parg) == 0) {
+	    parg++;
+	    if (*parg == 0)
+		fatal("Missing argument to -h");
+	    db_home = *parg;
+	}
+    }
+    if (opt_verbose)
+        syslog(LOG_NOTICE, "daemon started");
     rc = initialize();
     if (rc) {
 	errno = rc;
@@ -447,9 +453,10 @@ main(int argc, char **argv)
 		s = strchr(state, '\n');
 		*s = 0;
 	    }
-	    msg_info("Ignoring protocol state %s",
-		     protocol ? protocol : "(not defined)",
-		     state ? state : "(not defined)");
+	    syslog(LOG_DEBUG,
+		   "Ignoring protocol %s state %s",
+		   protocol ? protocol : "(not defined)",
+		   state ? state : "(not defined)");
 	    if (p)
 		*p = '\n';
 	    if (s)
@@ -460,7 +467,8 @@ main(int argc, char **argv)
 	safe_writes(1, "\n\n");
     }
     cleanup();
-    if (msg_verbose)
-	msg_info("daemon stopped");
+    if (opt_verbose)
+	syslog(LOG_NOTICE, "daemon stopped");
+    closelog();
     return 0;
 }
